<?php
/**
* Author:
* Bernard Szlachta
* bernard.szlachta@nobleprog.com
* www.nobleprog.co.uk
*
* 
* This software is GPL Licened
**/


/* 
	Exmaple usage
    function runquery($sql) {
	global $active_db;
	
	$cache_filename = sys_get_temp_dir() . md5($sql) . ".querycache";
	echo $cache_filename;
	$cache = & new Cache($active_db, getCacheQueryExpiryTime($sql), $cache_filename);
	$result = $cache->query($sql);
	return $result;
	}

 */

class Cache {
	private $mysql; // instance of MySQL object
	private $result; // instance of Result object
	private $expiry; // cache expire time in seconds
	private $cacheFile; // cache file
	private $data; // result set array
	private $query; // DEBUG purpouse only, after testing, comment it

	public $num_rows; //only for compatibility with mysqli

	function Cache(& $mysql, $expiry = 5, $cacheFile ) {

		$this->mysql = & $mysql;
		$this->cacheFile = $cacheFile;
		$this->data = array ();
		$this->expiry = $expiry;
	}

	function query($query) {
		//	echo $query;
		$this->query = $query;

		if (!preg_match("/^select/", trim(strtolower($query))) // not DQL
		or $this->expiry <= 0) { // not cached queries
			// It is not very elegant, but I didn't wanted to rewriting all functionality

			//$this->result = mysqli_query($this->mysql,$query . " /* not cached */");

			$this->result = $this->mysql->query($query . " /* not cached */");
			return $this->result;
		}
		if (!$this->isValid()) {
			$this->result = $this->mysql->query($query . "/* caching */");
		
			$this->data = $this->write();
			$this->count_rows();
		} else {
			$this->data = $this->read();
			$this->count_rows();
		}

		return $this;
	}

	function write() {
		$fp = fopen($this->cacheFile, 'w');

		flock($fp, LOCK_EX);

		while ($row = mysqli_fetch_array($this->result,MYSQL_BOTH)) {
			$content[] = $row;
		}

		fwrite($fp, serialize($content));

		flock($fp, LOCK_UN);
		fclose($fp);
		unset ($fp, $row);
		return $content;
	}

	function read() {
		$content = unserialize(file_get_contents($this->cacheFile));
		return $content;
	}

	function isValid() {
		return (file_exists($this->cacheFile) && filemtime($this->cacheFile) > (time() - $this->expiry));
	}

	function fetch_row() {
		if (!$this->data)
			return false;

		if (!$row = current($this->data)) {
			return false;
		}
		next($this->data);
		return $row;
	}

	function fetch_assoc() {
		return $this->fetch_row();
	}
	
	function fetch_array() {
		return $this->fetch_row();
	}

	function count_rows() {
		$rows = count($this->data);
		$this->num_rows = $rows;
		return $rows;
	}

	function fetch_object() {
		$assoc = $this->fetch_assoc();
		return $this->array2object($assoc); 
	}

/**
 * Helper function which changes array into object
 */
	private function array2object($array) {	
	    if (is_array($array)) {
	        $obj = new StdClass();
	        
	        foreach ($array as $key => $val){
	            $obj->$key = $val;
	        }
	    }
	    else { 
	    	$obj = $array; 
	    }	    
	    return $obj;
	}
} // End of the class


